/**
 * abssqr.com Inc. 
 * Copyright (c) 2017-2020 All Rights Reserved. 
 */
package com.abssqr.plat.common.model.utils;

import com.abssqr.plat.common.facade.enums.StageChangeTypeEnum;
import com.abssqr.plat.common.facade.model.plan.PlanAst;
import com.abssqr.plat.common.model.domain.plan.PlanAstLog;
import com.abssqr.plat.common.model.exception.AbssqrBizException;
import com.abssqr.plat.common.util.base.autoconfig.AbsConfig;
import com.abssqr.plat.common.util.utils.BigDecimalUtil;
import com.general.system.common.model.Money;
import com.general.system.common.util.LogUtil;
import com.general.system.common.util.MessageUtil;
import com.general.system.common.util.MoneyUtil;
import com.general.system.tool.util.SpringContextHolder;
import com.general.system.tool.util.ToolUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.DependsOn;
import org.springframework.stereotype.Component;

import java.util.List;

/**
 * 计划水位变更金额校验
 * @author hanfei
 * @version $Id: PlanAstChangeCheckerUtil.java, v 0.1 2020-01-06 4:24 PM hanfei Exp $
 */
@Component
@DependsOn("springContextHolder")
public class PlanAstChangeCheckerUtil {
    
    private static Logger LOGGER = LoggerFactory.getLogger(PlanAstChangeCheckerUtil.class);
    @Autowired
    private AbsConfig absConfig;
    
    public static PlanAstChangeCheckerUtil me() {
        try {
            return SpringContextHolder.getBean(PlanAstChangeCheckerUtil.class);
        }catch (Throwable e){
            return new PlanAstChangeCheckerUtil();
        }
    }
    
    /**
     * 分别对待结算金额，在途金额，冻结和水位金额进行前后值发生额相等计算；
     * @param planAstLogs
     */
    public void checkChangeLogAmt(List<PlanAstLog> planAstLogs){
        planAstLogs.forEach(one -> {
            //1、水位金额校验 cash与lastCash不相等，lastCash - cash 得出绝对值与cashAmt判等
            checkAmt(one.getPlanNo(), one.getLastCash(), one.getCash(), one.getCashAmt(), one.getChangeType());
            
            //2、冻结金额校验 frozenCash与lastFrozenCash不相等，lastFrozenCash- frozenCash得出绝对值与frozenCashAmt判等
            checkAmt(one.getPlanNo(), one.getLastFrozenCash(), one.getFrozenCash(), one.getFrozenCashAmt(), one.getChangeType());
            
            //3、累计投放金额校验 accGrant与lastAccGrant不相等，lastAccGrant- accGrant得出绝对值与accGrantAmt判等
            checkAmt(one.getPlanNo(), one.getLastAccGrant(), one.getAccGrant(), one.getAccGrantAmt(), one.getChangeType());
            
            /**
             * 4、在途金额校验
             payOnWay与lastPayOnWay不相等，lastPayOnWay- payOnWay得出绝对值与happenAmt判等
             repoOnWay与lastRepoOnWay不相等，lastRepoOnWay- repoOnWay得出绝对值与happenAmt判等
             */
            checkAmt(one.getPlanNo(), one.getLastPayOnWay(), one.getPayOnWay(), one.getHappenAmt(), one.getChangeType());
            checkAmt(one.getPlanNo(), one.getLastRepoOnWay(), one.getRepoOnWay(), one.getHappenAmt(), one.getChangeType());
        });
    }
    
    /**
     * 金额校验
     * @param lastAmt
     * @param nowAmt
     * @param hapAmt
     * @param changeType
     */
    private void checkAmt(String planNo, Money lastAmt, Money nowAmt,
                                 Money hapAmt, StageChangeTypeEnum changeType){
        if (ToolUtil.isOneEmpty(lastAmt, nowAmt, hapAmt)){
            errorHandle(MessageUtil.formatMsg(
                    "水位变更异常：[{0} - {1}]金额中有null,[lastAmt:{2}-nowAmt:{3}-hapAmt:{4}]",
                    planNo, changeType.getDesc(), lastAmt, nowAmt, hapAmt));
        }
        // 金额相等不校验
        if (MoneyUtil.compare(lastAmt, nowAmt) == 0){
            return;
        }
        // 获取差异金额
        Money amt = lastAmt.subtract(nowAmt);
        // 差异金额取绝对值与 发生额比较是否相等
        if (!BigDecimalUtil.isEQ(amt.getAmount().abs(), hapAmt.getAmount())){
            errorHandle(MessageUtil.formatMsg(
                    "水位变更异常：[{0} - {1}]变更金额与发生额不想等,[lastAmt:{2}-nowAmt:{3}-hapAmt:{4}]",
                    planNo, changeType.getDesc(), lastAmt, nowAmt, hapAmt));
        }
    }
    
    /**
     * 校验水位金额不能小于0
     * @param planAst
     */
    public void checkPlanAstAmtZero(PlanAst planAst){
        //1、水位金额cash校验不能小于0;
        checkGreaterThanZero(planAst.getPlanNo(), "水位余额", planAst.getCash());
        checkGreaterThanZero(planAst.getPlanNo(), "水位校验金额", planAst.getCheckCash());
        //2、冻结金额frozenCash校验不能小于0;
        checkGreaterThanZero(planAst.getPlanNo(), "冻结金额", planAst.getFrozenCash());
        //3、在途金额payOnWay,repoOnWay校验不能小于0;
        checkGreaterThanZero(planAst.getPlanNo(), "支出在途", planAst.getPayOnWay());
        checkGreaterThanZero(planAst.getPlanNo(), "收入在途", planAst.getRepoOnWay());
        //4、待结算金payClear,repoClear额校验不能小于0;
        checkGreaterThanZero(planAst.getPlanNo(), "支出待结算", planAst.getPayClear());
        checkGreaterThanZero(planAst.getPlanNo(), "收入待结算", planAst.getRepoClear());
    }
    
    private void checkGreaterThanZero(String planNo, String des, Money amt){
        if (amt.isNegtive()){
            throw new AbssqrBizException(MessageUtil.formatMsg(
                    "水位变更异常：[{0} - {1}]金额不能小于0,[amt:{2}]", planNo, des, amt));
        }
    }
    
    /**
     * 每笔操作对PlanAst 操作变更后
     * 通过计算公式： cash - payOnway - payClear + repoOnWay + repoClear = checkAmt  判断相等；
     */
    public void checkEqualFund(PlanAst planAst, PlanAstLog astLog) {
        // 发生额
        Money hmt = planAst.getCash().subtract(planAst.getPayOnWay()).subtractFrom(planAst.getPayClear())
                            .addTo(planAst.getRepoOnWay()).addTo(planAst.getRepoClear());
        if (MoneyUtil.compare(hmt, planAst.getCheckCash()) != 0){
            errorHandle(MessageUtil.formatMsg(
                    "水位变更异常：[{0}]计划处理交易[{4}-{5}],水位计算金额[{1}]与校验金额[{2}]不相等,发生额[{3}],编号[{6}]",
                    planAst.getPlanNo(), hmt, planAst.getCheckCash(),
                    astLog.getHappenAmt(), astLog.getTransCode(), astLog.getChangeType(), astLog.getTransBizNo()));
        }
    }
    
    /**
     * 异常处理
     * @param msg
     */
    private void errorHandle(String msg){
        if (absConfig.isAstCheckException()){
            throw new AbssqrBizException(msg);
        }else{
            LogUtil.error(LOGGER, msg);
        }
    }
}
